@@ -33,6 +33,7 @@ import yaml
 
 DEBUGGING = False
 UNPACK_DIR = None
+RAW_UNPACK_DIR = None
 
 # cleanup
 import atexit
@@ -42,6 +43,9 @@ def cleanup_unpack():
     global UNPACK_DIR
     if UNPACK_DIR is not None and os.path.isdir(UNPACK_DIR):
         recursive_rm(UNPACK_DIR)
+    global RAW_UNPACK_DIR
+    if RAW_UNPACK_DIR is not None and os.path.isdir(RAW_UNPACK_DIR):
+        recursive_rm(RAW_UNPACK_DIR)
 atexit.register(cleanup_unpack)
 
 
@@ -89,7 +93,6 @@ class ClickReview(object):
                        "frameworks",
                        "icon",
                        "immutable-config",
-                       "integration",
                        "oem",
                        "services",
                        "source",
@@ -126,6 +129,11 @@ class ClickReview(object):
             UNPACK_DIR = unpack_click(fn)
         self.unpack_dir = UNPACK_DIR
 
+        global RAW_UNPACK_DIR
+        if RAW_UNPACK_DIR is None:
+            RAW_UNPACK_DIR = raw_unpack_pkg(fn)
+        self.raw_unpack_dir = RAW_UNPACK_DIR
+
         # Get some basic information from the control file
         control_file = self._extract_control_file()
         tmp = list(Deb822.iter_paragraphs(control_file))
@@ -209,6 +217,29 @@ class ClickReview(object):
             return None  # snappy packaging is still optional
         return open_file_read(y)
 
+    def _extract_hashes_yaml(self):
+        '''Extract and read the snappy hashes.yaml'''
+        y = os.path.join(self.unpack_dir, "DEBIAN/hashes.yaml")
+        return open_file_read(y)
+
+    def _extract_statinfo(self, fn):
+        '''Extract statinfo from file'''
+        try:
+            st = os.stat(fn)
+        except Exception:
+            return None
+        return st
+
+    def _path_join(self, dirname, rest):
+        return os.path.join(dirname, rest)
+
+    def _get_sha512sum(self, fn):
+        '''Get sha512sum of file'''
+        (rc, out) = cmd(['sha512sum', fn])
+        if rc != 0:
+            return None
+        return out.split()[0]
+
     def _check_path_exists(self):
         '''Check that the provided path exists'''
         if not os.path.exists(self.click_package):
@@ -328,9 +359,6 @@ class ClickReview(object):
                 elif f in ["binaries", "services"] and not \
                         isinstance(self.pkg_yaml[f], list):
                     error("yaml malformed: '%s' is not list:\n%s" % (f, yp))
-                elif f == "integration" and not isinstance(self.pkg_yaml[f],
-                                                           dict):
-                    error("yaml malformed: '%s' is not dict:\n%s" % (f, yp))
                 elif f in ["icon", "source", "type", "vendor"] and not \
                         isinstance(self.pkg_yaml[f], str):
                     error("yaml malformed: '%s' is not str:\n%s" % (f, yp))
@@ -379,7 +407,12 @@ class ClickReview(object):
 
     def _verify_pkgname(self, n):
         '''Verify package name'''
-        if re.search(r'^[a-z0-9][a-z0-9+.-]+$', n):
+        if self.is_snap:
+            # snaps can't have '.' in the name
+            pat = re.compile(r'^[a-z0-9][a-z0-9+-]+$')
+        else:
+            pat = re.compile(r'^[a-z0-9][a-z0-9+.-]+$')
+        if pat.search(n):
             return True
         return False
 
@@ -612,6 +645,37 @@ def unpack_click(fn, dest=None):
     return dest
 
 
+def raw_unpack_pkg(fn, dest=None):
+    '''Unpack raw package'''
+    if not os.path.isfile(fn):
+        error("Could not find '%s'" % fn)
+    pkg = fn
+    if not pkg.startswith('/'):
+        pkg = os.path.abspath(pkg)
+
+    if dest is not None and os.path.exists(dest):
+        error("'%s' exists. Aborting." % dest)
+
+    d = tempfile.mkdtemp(prefix='review-')
+
+    curdir = os.getcwd()
+    os.chdir(d)
+    (rc, out) = cmd(['ar', 'x', pkg])
+    os.chdir(curdir)
+
+    if rc != 0:
+        if os.path.isdir(d):
+            recursive_rm(d)
+        error("'ar x' failed with '%d':\n%s" % (rc, out))
+
+    if dest is None:
+        dest = d
+    else:
+        shutil.move(d, dest)
+
+    return dest
+
+
 def open_file_read(path):
     '''Open specified file read-only'''
     try:
