@@ -20,9 +20,11 @@ from debian.deb822 import Deb822
 import glob
 import os
 import re
+import stat
+import yaml
 
 from clickreviews.frameworks import Frameworks
-from clickreviews.cr_common import ClickReview, open_file_read, cmd
+from clickreviews.cr_common import ClickReview, open_file_read, cmd, error
 
 CONTROL_FILE_NAMES = ["control", "manifest", "preinst"]
 MINIMUM_CLICK_FRAMEWORK_VERSION = "0.4"
@@ -251,7 +253,6 @@ class ClickReviewLint(ClickReview):
                 s = "Maintainer=%s does not match manifest maintainer=%s" % \
                     (control['Maintainer'], self.manifest['maintainer'])
         else:
-            t = 'warn'
             s = 'Skipped: maintainer not in manifest'
         self._add_result(t, n, s)
 
@@ -495,7 +496,9 @@ exit 1
         self._add_result(t, n, s)
 
     def check_pkgname(self):
-        '''Check package name valid'''
+        '''Check click package name valid'''
+        if self.is_snap:
+            return
         p = self.manifest['name']
         # http://www.debian.org/doc/debian-policy/ch-controlfields.html
         t = 'info'
@@ -572,8 +575,13 @@ exit 1
         n = 'maintainer_present'
         s = 'OK'
         if 'maintainer' not in self.manifest:
-            s = 'required maintainer field not specified in manifest'
-            self._add_result('error', n, s)
+            if self.is_snap:
+                s = 'Skipped optional maintainer field not specified in ' + \
+                    'manifest'
+            else:
+                t = 'error'
+                s = 'required maintainer field not specified in manifest'
+            self._add_result(t, n, s)
             return
         self._add_result(t, n, s)
 
@@ -1031,7 +1039,7 @@ exit 1
         return contents
 
     def check_snappy_readme_md(self):
-        '''Check package architecture in package.yaml is valid'''
+        '''Check snappy readme.md'''
         if not self.is_snap:
             return
 
@@ -1055,3 +1063,224 @@ exit 1
             t = 'warn'
             s = "meta/readme.md is too short"
         self._add_result(t, n, s)
+
+    def _check_innerpath_executable(self, fn):
+        '''Check that the provided path exists and is executable'''
+        return os.access(fn, os.X_OK)
+
+    def check_snappy_config(self):
+        '''Check snappy config'''
+        if not self.is_snap:
+            return
+
+        fn = os.path.join(self.unpack_dir, 'meta/hooks/config')
+        if fn not in self.pkg_files:
+            return
+
+        t = 'info'
+        n = 'snappy_config_hook_executable'
+        s = 'OK'
+        if not self._check_innerpath_executable(fn):
+            t = 'error'
+            s = 'meta/hooks/config is not executable'
+        self._add_result(t, n, s)
+
+    def check_snappy_services_and_binaries(self):
+        '''Services and binaries should not overlap'''
+        if not self.is_snap:
+            return
+        for exe_t in ['binaries', 'services']:
+            if exe_t not in self.pkg_yaml:
+                break
+            if exe_t == 'binaries':
+                other_exe_t = 'services'
+            else:
+                other_exe_t = 'binaries'
+
+            if other_exe_t not in self.pkg_yaml:
+                break
+
+            for a in self.pkg_yaml[exe_t]:
+                if 'name' not in a:
+                    continue
+                # handle 'exec' in binaries
+                app = os.path.basename(a['name'])
+
+                t = 'info'
+                n = 'snappy_%s_in_%s' % (app, other_exe_t)
+                s = 'OK'
+                for other_a in self.pkg_yaml[other_exe_t]:
+                    other_app = os.path.basename(other_a['name'])
+                    if app == other_app:
+                        t = 'error'
+                        s = "'%s' in both 'services' and 'binaries'" % app
+                        break
+                self._add_result(t, n, s)
+
+    def check_snappy_hashes(self):
+        '''Check snappy hashes.yaml'''
+        if not self.is_snap:
+            return
+
+        def _check_allowed_perms(mode, allowed):
+            '''Check that mode only uses allowed perms'''
+            for p in mode[1:]:
+                if p not in allowed:
+                    return False
+            return True
+
+        try:
+            hashes_yaml = yaml.safe_load(self._extract_hashes_yaml())
+        except Exception:
+            error("Could not load hashes.yaml. Is it properly formatted?")
+
+        if 'archive-sha512' not in hashes_yaml:
+            t = 'error'
+            n = 'hashes_archive-sha512_present'
+            s = "'archive-sha512' not found in hashes.yaml"
+            self._add_result(t, n, s)
+            return
+
+        # verify the ar file
+        t = 'info'
+        n = 'hashes_archive-sha512_valid'
+        s = 'OK'
+        fn = self._path_join(self.raw_unpack_dir, 'data.tar.gz')
+        sum = self._get_sha512sum(fn)
+        if hashes_yaml['archive-sha512'] != sum:
+            t = 'error'
+            s = "hash mismatch: '%s' != '%s'" % (hashes_yaml['archive-sha512'],
+                                                 sum)
+            self._add_result(t, n, s)
+            return
+        self._add_result(t, n, s)
+
+        if 'files' not in hashes_yaml:
+            t = 'error'
+            n = 'hashes_files_present'
+            s = "'files' not found in hashes.yaml"
+            self._add_result(t, n, s)
+            return
+
+        # verify the individual files
+        errors = []
+        badsums = []
+        hash_files = set([])  # used to check with extra files
+        for entry in hashes_yaml['files']:
+            if 'name' not in entry:
+                errors.append("'name' not found for entry '%s'" % entry)
+                continue
+            else:
+                hash_files.add(entry['name'])
+
+            if 'mode' not in entry:
+                errors.append("'mode' not found for entry '%s'" %
+                              entry['name'])
+                continue
+            elif len(entry['mode']) != 10:
+                errors.append("malformed mode '%s' for entry '%s'" %
+                              (entry['mode'], entry['name']))
+                continue
+            elif entry['mode'].startswith('d'):
+                if not _check_allowed_perms(entry['mode'],
+                                            ['r', 'w', 'x', '-']):
+                    errors.append("unusual mode '%s' for entry '%s'" %
+                                  (entry['mode'], entry['name']))
+                # world write shouldn't be allowed
+                if entry['mode'][-2] != '-':
+                    errors.append("'%s' is world-writable" % entry['name'])
+                continue
+            elif entry['mode'].startswith('l'):
+                if entry['mode'] != "lrwxrwxrwx":
+                    errors.append("unusual mode '%s' for entry '%s'" %
+                                  (entry['mode'], entry['name']))
+                continue
+            elif not entry['mode'].startswith('f'):
+                # files and symlinks are ok, everything else is not
+                errors.append("illegal file mode '%s': '%s' for '%s'" %
+                              (entry['mode'][0], entry['mode'], entry['name']))
+                continue
+            elif 'size' not in entry:
+                errors.append("'size' not found for entry: %s" % entry['name'])
+                continue
+            elif 'sha512' not in entry:
+                errors.append("'sha512' not found for entry: %s" %
+                              entry['name'])
+                continue
+
+            fn = self._path_join(self.unpack_dir, entry['name'])
+
+            # quick verify the size, if it is wrong, we don't have to do the
+            # sha512sum
+            statinfo = self._extract_statinfo(fn)
+            # file is missing
+            if statinfo is None:
+                errors.append("'%s' does not exist" % entry['name'])
+                continue
+            if entry['size'] != statinfo.st_size:
+                errors.append("size %d != %d for '%s'" % (entry['size'],
+                                                          statinfo.st_size,
+                                                          entry['name']))
+                continue
+
+            # check for weird perms
+            if not _check_allowed_perms(entry['mode'], ['r', 'w', 'x', '-']):
+                errors.append("unusual mode '%s' for entry '%s'" %
+                              (entry['mode'], entry['name']))
+                continue
+            # world write shouldn't be allowed
+            if entry['mode'][-2] != '-':
+                errors.append("mode '%s' for '%s' is world-writable" %
+                              (entry['mode'], entry['name']))
+                continue
+            # verify perms match what is in hashes.yaml
+            filemode = stat.filemode(statinfo.st_mode)[1:]
+            if entry['mode'][1:] != filemode:
+                errors.append("mode '%s' != '%s' for '%s'" %
+                              (entry['mode'][1:], filemode, entry['name']))
+                continue
+
+            # ok, now all the cheap tests are done so we can check if we have a
+            # valid sha512sum
+            sum = self._get_sha512sum(fn)
+            if entry['sha512'] != sum:
+                badsums.append("'%s' != '%s' for '%s'" % (entry['sha512'], sum,
+                                                          entry['name']))
+
+        t = 'info'
+        n = 'sha512sums'
+        s = 'OK'
+        if len(badsums) > 0:
+            t = 'error'
+            s = 'found bad checksums: %s' % ", ".join(badsums)
+        self._add_result(t, n, s)
+
+        t = 'info'
+        n = 'file_mode'
+        s = 'OK'
+        if len(errors) > 0:
+            t = 'error'
+            s = 'found errors in hashes.yaml: %s' % ", ".join(errors)
+        self._add_result(t, n, s)
+
+        # Now check for extra files
+        t = 'info'
+        n = 'hashes_extra_files'
+        s = 'OK'
+        self._add_result(t, n, s)
+        extra = []
+        click_compat_files = set(["DEBIAN/hashes.yaml",
+                                  "DEBIAN/control",
+                                  "DEBIAN/preinst",
+                                  "DEBIAN/manifest"
+                                  ])
+        for f in self.pkg_files:
+            fn = os.path.relpath(f, self.unpack_dir)
+            if fn not in hash_files and fn not in click_compat_files:
+                extra.append(fn)
+
+        if len(extra) > 0:
+            t = 'error'
+            s = 'found extra files not listed in hashes.yaml: %s' % \
+                ", ".join(extra)
+        self._add_result(t, n, s)
